@@ -18,6 +18,7 @@ import signal
 import atexit
 import random
 
+
 coloredlogs.DEFAULT_FIELD_STYLES['asctime'] = {}
 coloredlogs.DEFAULT_FIELD_STYLES['levelname'] = {'bold': True, 'color': 'white', 'bright': True}
 coloredlogs.DEFAULT_LEVEL_STYLES['info'] = {'color': 'cyan', 'bright': True}
@@ -37,8 +38,10 @@ __all__ = ["gdaAttack"]
 
 try:
     from .gdaTools import getInterpolatedValue, getDatabaseInfo
+    from .dupCheck import DupCheck
 except ImportError:
     from gdaTools import getInterpolatedValue, getDatabaseInfo
+    from dupCheck import DupCheck
 
 theCacheQueue = None
 theCacheThreadObject = None
@@ -171,6 +174,8 @@ class gdaAttack:
         self._attackCounter = 0
         self._claimCounter = 0
         self._guessCounter = 0
+        # State for duplicate claim detection
+        self._dupCheck = DupCheck()
         # State for computing attack results (see _initAtkRes())
         self._atrs = {}
         # State for various operational measures (see _initOp())
@@ -305,6 +310,9 @@ class gdaAttack:
         if doExit:
             sys.exit(exitMsg)
 
+    def isClaimed(self, spec):
+        return self._dupCheck.is_claimed(spec, verbose=self._vb)
+
     def askClaim(self, spec, cache=True, claim=True):
         """Generate Claim query for raw and optionally pub databases.
 
@@ -330,6 +338,8 @@ class gdaAttack:
         Answers are cached <br/>
         Returns immediately"""
         if self._vb: print(f"Calling {__name__}.askClaim with spec '{spec}', count {self._claimCounter}")
+        if not self._dupCheck.is_claimed(spec, verbose=self._vb, raise_true=True):
+            self._dupCheck.claim(spec, verbose=self._vb)
         self._claimCounter += 1
         sql = self._makeSqlFromSpec(spec)
         if self._vb: print(f"Sql is '{sql}'")
@@ -900,7 +910,7 @@ class gdaAttack:
                 if selectColumn in pair[0]:
                     colType = pair[1]
                     break
-            if colType == 'text' or 'date' in colType or 'time' in colType:
+            if 'text' in colType or 'char' in colType or 'date' in colType or 'time' in colType:
                 sql = initSql + str(f" FROM {table} WHERE {selectColumn} >= '{colRange[0]}' and {selectColumn} <= '{colRange[1]}'")
             else:
                 sql = initSql + str(f" FROM {table} WHERE {selectColumn} >= {colRange[0]} and {selectColumn} <= {colRange[1]}")
@@ -1373,7 +1383,7 @@ class gdaAttack:
         sqls = []
         numGuess = len(spec['guess'])
         if self._cr == 'inference' or self._cr == 'singlingOut':
-            sql = str(f"select count(*) from {self._p['table']} where ")
+            sql = str(f"select count(distinct {self._p['uid']}) from {self._p['table']} where ")
             # This first sql learns the number of rows matching the
             # guessed values
             for i in range(numGuess):
@@ -1384,7 +1394,7 @@ class gdaAttack:
             sqls.append(sql)
             # This second sql learns the total number of rows (should
             # normally be a cached result)
-            sql = str(f"select count(*) from {self._p['table']}")
+            sql = str(f"select count(distinct {self._p['uid']}) from {self._p['table']}")
             sqls.append(sql)
         elif self._cr == 'linkability':
             # nothing happens for linkability
